@@ -25,19 +25,28 @@ from cubicweb.selectors import is_instance
 
 from cubes.tracker import hooks as tracker
 
-TICKET_EID_RGX = re.compile('#(\d+)')
+_BASE_RE = r'\b%s\b\s+(#\d+(?:(?:,|\s|\s+and\s)\s*#\d+)*)'
+CLOSES_EID_RGX = re.compile(_BASE_RE % 'closes', re.IGNORECASE)
+RELATED_EID_RGX = re.compile(_BASE_RE % '(?:closes|related to)', re.IGNORECASE)
+TICKET_EID_RGX = re.compile(r'#(\d+)')
 
 def closed_eids(line):
-    words = [w.lower() for w in re.findall('#?\w+', line.strip())]
-    try:
-        idx = words.index('closes')
-    except ValueError:
-        return ()
-    matches = TICKET_EID_RGX.findall(' '.join(words[idx+1:]))
-    return [int(i) for i in matches]
+    """Parse closes command in description and return target eids"""
+    eids = []
+    for cmd in CLOSES_EID_RGX.findall(line):
+        eids.extend(int(m) for m in TICKET_EID_RGX.findall(cmd))
+    return eids
+
+def related_eids(line):
+    """Parse closes and related commands in description and return target eids
+    """
+    eids = []
+    for cmd in RELATED_EID_RGX.findall(line):
+        eids.extend(int(m) for m in TICKET_EID_RGX.findall(cmd))
+    return eids
 
 def tickets(session, ticket_eids, repository_eid):
-    """return rset for a ticket referenced in a commit message.
+    """Return rset for a ticket referenced in a commit message.
 
     The ticket must come from a project using given repository as
     source_repository (or one of its subproject).
@@ -72,7 +81,7 @@ WHERE NOT T closed_by R,
 '''
 class SearchRevisionTicketOp(hook.DataOperationMixIn,
                              hook.Operation):
-    """search magic words in revision's commit message:
+    """Search magic words in revision's commit message:
 
         closes #<ticket eid>
 
@@ -83,9 +92,7 @@ class SearchRevisionTicketOp(hook.DataOperationMixIn,
         for rev_eid in self.get_data():
             # search for instruction
             rev = self.session.entity_from_eid(rev_eid)
-            eids = []
-            for line in rev.description.splitlines():
-                eids.extend(closed_eids(line))
+            eids = closed_eids(rev.description)
             for ticket in tickets(self.session, eids, rev.repository.eid):
                 if rev.phase == 'public':
                     self.session.execute(_CLOSED_BY_RQL,
@@ -97,6 +104,10 @@ class SearchRevisionTicketOp(hook.DataOperationMixIn,
                             break
 
 class RevisionAdded(hook.Hook):
+    """Detect that a new revision is touched to prepare relevant operation
+
+    On update we ignore non-phase change.
+    """
     __regid__ = 'trackervcs.revision-to-ticket-state'
     __select__ = hook.Hook.__select__ & is_instance('Revision')
     events = ('after_add_entity', 'after_update_entity')
@@ -115,11 +126,14 @@ class RevisionAdded(hook.Hook):
 # "P has_apycot_environment PE AND PE local_repository R"
 
 class SetSourceRepositoryOp(hook.Operation):
+    """Automatically assign project repo to test env repo"""
     def precommit_event(self):
         if not self.project.source_repository and self.projectenv.repository:
-            self.project.set_relations(source_repository=self.projectenv.repository)
+            repo = self.projectenv.repository
+            self.project.set_relations(source_repository=repo)
 
 class ApycotEnvironmentAdded(hook.Hook):
+    """Automatically assign project repo to test env repo"""
     __regid__ = 'trackervcs.add-source-repository'
     __select__ = hook.Hook.__select__ & hook.match_rtype('has_apycot_environment')
     events = ('after_add_relation',)
@@ -134,6 +148,7 @@ class ApycotEnvironmentAdded(hook.Hook):
 # set ticket 'in-progress' when attached to a patch ############################
 
 class PatchLinkedToTicket(hook.Hook):
+    """Set ticket 'in-progress' when attached to a patch"""
     __regid__ = 'trackervcs.patch-linked-to-ticket'
     __select__ = hook.Hook.__select__ & hook.match_rtype('patch_ticket')
     events = ('after_add_relation',)
@@ -146,6 +161,7 @@ class PatchLinkedToTicket(hook.Hook):
 # attach patch to ticket on addition ###########################################
 
 class PatchRevisionAdded(hook.Hook):
+    """Attach patch to ticket on addition"""
     __regid__ = 'trackervcs.patch-revision-added'
     __select__ = hook.Hook.__select__ & hook.match_rtype('patch_revision')
     events = ('after_add_relation',)
@@ -155,9 +171,7 @@ class PatchRevisionAdded(hook.Hook):
         patch = self._cw.entity_from_eid(self.eidfrom)
         rev = self._cw.entity_from_eid(self.eidto)
         source_repo = rev.repository
-        eids = []
-        for line in rev.description.splitlines():
-            eids.extend(closed_eids(line))
+        eids = related_eids(rev.description)
         for ticket in tickets(self._cw, eids, source_repo.eid):
             # XXX should we drop existing links first?
             # We could just run the RQL below if there wasn't a cache issue
