@@ -31,14 +31,14 @@ class introduced_by(RelationDefinition):
     subject = 'Ticket'
     object = 'Revision'
     constraints = [RQLConstraint('S concerns P, O from_repository R, '
-                                 'P source_repository R')]
+                                 'EXISTS(P source_repository R) OR EXISTS(P subproject_of PP, PP source_repository R)')]
 
 
 class closed_by(RelationDefinition):
     subject = 'Ticket'
     object = 'Revision'
     constraints = [RQLConstraint('S concerns P, O from_repository R, '
-                                 'P source_repository R')]
+                                 'EXISTS(P source_repository R) OR EXISTS(P subproject_of PP, PP source_repository R)')]
 
 if 'Patch' in context.defined:
     from cubes.vcreview.schema import Patch
@@ -50,7 +50,7 @@ if 'Patch' in context.defined:
         constraints = [RQLConstraint('S patch_revision RE, '
                                      'RE from_repository R, '
                                      'O concerns P, '
-                                     'P source_repository R')]
+                                     'EXISTS(P source_repository R) OR EXISTS(P subproject_of PP, PP source_repository R)')]
 
 
     class require_permission(RelationDefinition):
